import pandas as pd


def combine_forecast_ground_truth(forecast_df: pd.DataFrame, ground_truth_df: pd.DataFrame) ->pd.DataFrame:
    """
    Combine the forecast results with the ground truth (ts, id, horizon (in hours), pred, truth, diff)


    forecast_df should have the following columns
    - timestamp
    - pv_id
    - horizon_hours
    - power

    ground_truth_df should have the following columns
    - timestamp
    - pv_id
    - horizon_hours
    - power

    returns a dataframe with the following columns
    - timestamp
    - pv_id
    - horizon_hours
    - forecast_power
    - generation_power

    """

    # rename power to forecast_power
    forecast_df = forecast_df.rename(columns={"power_wh": "forecast_power"})

    # rename power to ground_truth_power
    ground_truth_df = ground_truth_df.rename(columns={"value": "generation_power"})

    # make pv_ids are ints
    forecast_df["pv_id"] = forecast_df["pv_id"].astype(int)
    ground_truth_df["pv_id"] = ground_truth_df["pv_id"].astype(int)

    # merge the two dataframes
    combined_df = pd.merge(forecast_df, ground_truth_df, on=["timestamp", "pv_id", "horizon_hour"])

    return combined_df


def make_hf_filename(timestamp_floor):
    """
    Make ICON filename from timestamp_floor

    '2021-01-01 00:00:00' ->
    'zip:///::hf://datasets/openclimatefix/dwd-icon-eu/data/2021/1/1/20210101_00.zarr.zip'

    """
    year = timestamp_floor.year
    month = timestamp_floor.month
    day = timestamp_floor.day
    date_and_hour = timestamp_floor.strftime("%Y%m%d_%H")
    date = f"{year}/{month}/{day}"
    file_location = f"{date}/{date_and_hour}"
    huggingface_route = "zip:///::hf://datasets/openclimatefix/dwd-icon-eu/data"
    # huggingface_route = "datasets/openclimatefix/dwd-icon-eu/data"
    huggingface_file = f"{huggingface_route}/{file_location}.zarr.zip"
    return date_and_hour, huggingface_file



import pandas as pd
import xarray as xr
import os
from psp.data_sources.nwp import NwpDataSource
from psp.data_sources.pv import NetcdfPvDataSource
from psp.serialization import load_model
from psp.typings import X

dir_path = os.path.dirname(os.path.realpath(__file__))


def forecast_v1(nwp_source:str, nwp_xr:xr.Dataset, pv_xr:xr.Dataset, ts:pd.Timestamp, model=None):
    """
    Run the forecast

    This runs the pv-site-prediction model from the psp library.
    """

    if model is None:
        model = load_model(f"{dir_path}/../models/model-0.3.0.pkl")

    # format pv and nwp data
    pv_data_source = NetcdfPvDataSource(
        pv_xr,
        id_dim_name="pv_id",
        timestamp_dim_name="timestamp",
        rename={"generation_wh": "power", "kwp": "capacity"},
        ignore_pv_ids=[],
    )
    # make NwpDataSource
    nwp = NwpDataSource(nwp_xr, value_name=nwp_source)
    model.set_data_sources(pv_data_source=pv_data_source, nwp_data_sources={nwp_source: nwp})

    # make prediction.
    # Note pv_id=1 is arbitrary, but the pv_xr must have this in it.
    x = X(pv_id="1", ts=ts)
    pred = model.predict(x)

    # format into timerange and put into pd dataframe
    times = pd.date_range(start=x.ts, periods=len(pred.powers), freq="15min")
    pred_df = pd.DataFrame({"power_wh": pred.powers}, index=times)

    return pred_df



import numpy as np
import pandas as pd


def metrics(results_df: pd.DataFrame, pv_metadata: pd.DataFrame):
    """
    Calculate and print metrics: MAE

    results_df dataframe with the following columns
    - timestamp
    - pv_id
    - horizon_hours
    - forecast_power
    - generation_power

    pv_metadata is a dataframe with the following columns
    - pv_id
    - capacity

    """

    # merge pv_metadata with results_df
    results_df = pd.merge(results_df, pv_metadata, on="pv_id")

    mae = np.round((results_df["forecast_power"] - results_df["generation_power"]).abs().mean(), 4)
    mae_normalized = np.round(
        ((results_df["forecast_power"] - results_df["generation_power"]) / results_df["capacity"])
        .abs()
        .mean(),
        4,
    )
    print(f"MAE: {mae} kw, normalized {mae_normalized} %")

    # calculate metrics over the different horizons hours
    # find all unique horizon_hours
    horizon_hours = results_df["horizon_hour"].unique()
    for horizon_hour in horizon_hours:
        # filter results_df to only include the horizon_hour
        results_df_horizon = results_df[results_df["horizon_hour"] == horizon_hour]
        mae = np.round(
            (results_df_horizon["forecast_power"] - results_df_horizon["generation_power"])
            .abs()
            .mean(),
            3,
        )
        sem = np.round(
            (
                (results_df_horizon["forecast_power"] - results_df_horizon["generation_power"])
                .abs()
                .std()
                / 50 ** 0.5
            ),
            3,
        )
        mae_normalized = np.round(
            (
                (results_df_horizon["forecast_power"] - results_df_horizon["generation_power"])
                / results_df_horizon["capacity"]
            )
            .abs()
            .mean(),
            3,
        )

        print(
            f"MAE for horizon {horizon_hour}: {mae} +- {1.96*sem}. Normalized MAE: {mae_normalized} %"
        )

    # calculate metrics over the different horizon groups
    horizon_groups = [[0, 0], [1, 1], [2, 2], [3, 4], [5, 8], [9, 16], [17, 24], [24, 48]]
    for horizon_group in horizon_groups:
        horizon_group_df = results_df[
            results_df["horizon_hour"].between(horizon_group[0], horizon_group[1])
        ]
        mae = np.round(
            (horizon_group_df["forecast_power"] - horizon_group_df["generation_power"])
            .abs()
            .mean(),
            3,
        )
        sem = np.round(
            (
                (horizon_group_df["forecast_power"] - horizon_group_df["generation_power"])
                .abs()
                .std()
                / 50 ** 0.5
            ),
            3,
        )

        mae_normalized = np.round(
            (
                (horizon_group_df["forecast_power"] - horizon_group_df["generation_power"])
                / horizon_group_df["capacity"]
            )
            .abs()
            .mean(),
            3,
        )

        print(
            f"MAE for horizon {horizon_group}: {mae} +- {1.96*sem}. mae_normalized: {mae_normalized} %"
        )

        # TODO add more metrics using ocf_ml_metrics
